package hero.entity;

import hero.interfaces.BnNodeHookPK;
import hero.interfaces.BnNodeHookValue;
import hero.interfaces.BnNodeLocal;
import hero.interfaces.InvalidValueException;
import hero.interfaces.SequenceGenerator;
import hero.interfaces.SequenceGeneratorHome;
import hero.interfaces.ServiceUnavailableException;

import java.rmi.RemoteException;

import javax.ejb.CreateException;
import javax.ejb.EJBException;
import javax.ejb.EntityBean;
import javax.ejb.EntityContext;
import javax.ejb.RemoveException;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.rmi.PortableRemoteObject;

/**
 * The Entity bean represents a BnProjectHook
 *
 * @author Oriol Montala
 * @version $Revision: 0.1 $
 *
 * @ejb:bean name="BnNodeHook"
 *           display-name="BnNode Hook Entity"
 *           type="CMP"
 *           jndi-name="ejb/hero/BnNodeHook"
 *           local-jndi-name="ejb/hero/BnNodeHook_L"
 *           view-type="both"
 *           cmp-version="2.x"
 *           reentrant="true"    
 *
 * @ejb:env-entry name="SequenceName"
 *                value="BnNodeHook"
 *
 * @ejb:ejb-ref ejb-name="hero/SequenceGenerator"
 *
 * @ejb:transaction type="Supports"
 *
 * @ejb.pk generate="true" method-intf="both"
 * 
 * @jonas.bean
 *     ejb-name="BnNodeHook"
 *     jndi-name="ejb/hero/BnNodeHook"
 * @jonas.jdbc-mapping
 *     jndi-name="jdbc_1"
 *     jdbc-table-name="BnNodeHook"
 * @jonas.cleanup create
 *
 * @ejb:finder signature="java.util.Collection findAll()" 
 *     query="SELECT object(n) FROM BnNodeHook n"
 *     transaction-type="Supports"
 *
 * @ejb:finder signature="java.util.Collection findByNode(int nodeId)" view-type="local" 
 *     query="SELECT OBJECT(p) From BnNodeHook p Where p.bnNode.id = ?1"
 *     transaction-type="Supports"
 * @ejb:finder signature="hero.interfaces.NodeHookLocal findByName(int nodeId,java.lang.String name)" view-type="local" 
 *     query="SELECT OBJECT(p) From BnNodeHook p Where p.bnNode.id = ?1 AND p.name = ?2"
 *     transaction-type="Supports"
 *
 *
 * @ejb.value-object
 *    match="*"
 *    name="BnNodeHook"
 *
 *
 * @jboss:table-name table-name="BnNodeHook"
 *
 * @jboss:create-table create="true"
 *
 * @jboss:remove-table remove="false"
 **/
public abstract class BnNodeHookBean
   implements EntityBean
{
   
   // -------------------------------------------------------------------------
   // Members
   // -------------------------------------------------------------------------  
   
   public EntityContext mContext;
   
   // -------------------------------------------------------------------------
   // Methods
   // -------------------------------------------------------------------------     
   

   // -------------------------------------------------------------------------
   // Properties (Getters/Setters)
   // -------------------------------------------------------------------------  
   
   /**
   * Retrieve the BnNodeHook's id.
   *
   * @return Returns an int representing the id of this BnNodeHook.
   *
   * @ejb:pk-field
   *
   * @ejb:persistence column-name="id"
   * @jonas.cmp-field-jdbc-mapping
   *    field-name="id"
   *    jdbc-field-name="id"
   * @ejb:transaction type="Supports"
   **/
   public abstract int getId();
   
   /**
   * Set the Hook's id.
   *
   * @param pId The id of this Hook. Is set at creation time.
   * @ejb:transaction type="Required"
   **/
   public abstract void setId( int pId );
   
   
    /** Retrieve the BnNode.
     *
     * @return Returns a BnNodeLocal.
     * @ejb:interface-method view-type="both"
     * @ejb:relation name="nodehook" role-name="hook-by" cascade-delete="yes"
     * @jboss:auto-key-fields
     * @jonas:automatic-pk="true"
     * @ejb:transaction type="Supports"
     **/
    public abstract BnNodeLocal getBnNode();
    
    /**
     * Set the BnNode.
     *
     * @ejb:interface-method view-type="local"
     * @param pNode The BnNode of this Hook.
     * @ejb:transaction type="Required"
     **/
    public abstract void setBnNode( BnNodeLocal pNode );



   /**
   * Retrieve the Name value.
   *
   * @return Returns an String representing the Hook Name value.
   *
   * @ejb:interface-method view-type="both"
   *
   * @ejb:persistence column-name="name"
   * @ejb:transaction type="Supports"
   **/
   public abstract String getName();
   
   /**
   * Set the Name value.
   * @ejb:interface-method view-type="both"
   * @param key The Name of this Hook.
   * @ejb:transaction type="Required"
   **/
   public abstract void setName( String hookName );

   /**
   * Retrieve the Event value.
   *
   * @return Returns an String representing the Event value.
   *
   * @ejb:interface-method view-type="both"
   *
   * @ejb:persistence column-name="event"
   * @ejb:transaction type="Supports"
   **/
   public abstract String getEvent();
   
   /**
   * Set the Event value.
   * @ejb:interface-method view-type="both"
   * @param value The Event value of this Hook.
   * @ejb:transaction type="Required"
   **/
   public abstract void setEvent( String hookEvent );


   /**
   * Retrieve the Type value.
   *
   * @return Returns an int representing the Type value.
   *
   * @ejb:interface-method view-type="both"
   *
   * @ejb:persistence column-name="type"
   * @ejb:transaction type="Supports"
   **/
   public abstract int getType();
   
   /**
   * Set the Type value.
   * @ejb:interface-method view-type="both"
   * @param value The Type value of this Hook.
   * @ejb:transaction type="Required"
   **/
   public abstract void setType( int hookType );

    /**
     * @ejb.interface-method 
     * @ejb:transaction type="Supports"
     */
    public abstract BnNodeHookValue getBnNodeHookValue();
    
    /**
     * @ejb.interface-method 
     * @ejb:transaction type="Required"
     */
    public abstract void setBnNodeHookValue(BnNodeHookValue v);



   // -------------------------------------------------------------------------
   // Framework Callbacks
   // -------------------------------------------------------------------------  
   
   /**
   * Create a Hook based on the supplied Hook Value Object.
   *
   *
   * @throws InvalidValueException If one of the values are not correct,
   *                               this will not roll back the transaction
   *                               because the caller has the chance to
   *                               fix the problem and try again
   * @throws EJBException If no new unique ID could be retrieved this will
   *                      rollback the transaction because there is no
   *                      hope to try again
   * @throws CreateException Because we have to do so (EJB spec.)
   *
   * @ejb:create-method view-type="both"
   **/
   public BnNodeHookPK ejbCreate(BnNodeHookValue prp )
      throws
         InvalidValueException,
         EJBException,
         CreateException
   {
       setBnNodeHookValue(prp);

      try {
         // Each title must have a unique id to identify itself within the DB
         this.setId( generateUniqueId() );
      }
      catch( ServiceUnavailableException se ) {
         // The unique id could not be set therefore terminate the transaction
         // by throwing a system exception
         throw new EJBException( se.getMessage() );
      }
      // This is only possible in CMPs. Otherwise return a valid PK.
      return null;
   }

   /**
   * Create a Hook based on the supplied  parameters.
   *
   *
   * @throws InvalidValueException If one of the values are not correct,
   *                               this will not roll back the transaction
   *                               because the caller has the chance to
   *                               fix the problem and try again
   * @throws EJBException If no new unique ID could be retrieved this will
   *                      rollback the transaction because there is no
   *                      hope to try again
   * @throws CreateException Because we have to do so (EJB spec.)
   *
   * @ejb:create-method view-type="both"
   **/
   public BnNodeHookPK ejbCreate(String hookName, String hookEvent, int hookType)
      throws
         InvalidValueException,
         EJBException,
         CreateException
   {
       this.setName(hookName);
       this.setEvent(hookEvent);
       this.setType(hookType);

      try {
         // Each title must have a unique id to identify itself within the DB
         this.setId( generateUniqueId() );
      }
      catch( ServiceUnavailableException se ) {
         // The unique id could not be set therefore terminate the transaction
         // by throwing a system exception
         throw new EJBException( se.getMessage() );
      }
      // This is only possible in CMPs. Otherwise return a valid PK.
      return null;
   }

   public void setEntityContext( EntityContext lContext )
   {
      mContext = lContext;
   }
   
   public void unsetEntityContext()
   {
      mContext = null;
   }
   
   public void ejbActivate()
   {
   }
   
   public void ejbPassivate()
   {
   }
   
   public void ejbLoad()
   {
   }
   
   public void ejbStore()
   {
   }
   
   public void ejbRemove()
      throws
         RemoveException
   {
   }


   /**
   * Retrive a unique creation id to use for this bean.  This will end up
   * demarcating this bean from others when it is stored as a record
   * in the database.
   *
   * @return Returns an integer that can be used as a unique creation id.
   *
   * @throws ServiceUnavailableException Indicating that it was not possible
   *                                     to retrieve a new unqiue ID because
   *                                     the service is not available
   **/
   private int generateUniqueId()
      throws ServiceUnavailableException
   {
      int lUniqueId = -1;
      try {
         Context lContext = new InitialContext();
         
         String lSequenceName = (String) lContext.lookup( 
            "java:comp/env/SequenceName" 
         );
         SequenceGeneratorHome lHome = (SequenceGeneratorHome) PortableRemoteObject.narrow(
            lContext.lookup(
               "java:comp/env/ejb/hero/SequenceGenerator"
            ),
            SequenceGeneratorHome.class
         );
         SequenceGenerator lBean = (SequenceGenerator) lHome.create();
         lUniqueId = lBean.getNextNumber( lSequenceName );
         lBean.remove();  
      }
      catch ( NamingException ne ) {
         throw new ServiceUnavailableException( "Naming lookup failure: " + ne.getMessage() );
      }
      catch ( CreateException ce ) {
         throw new ServiceUnavailableException( "Failure while creating a generator session bean: " + ce.getMessage() );
      }
      catch ( RemoveException re ) {
         // When the Bean cannot be removed after a while it will be taken back by the container
         // therefore ignore this exception
      }
      catch ( RemoteException rte ) {
         throw new ServiceUnavailableException( "Remote exception occured while accessing generator session bean: " +  rte.getMessage() );
      }
      
      return lUniqueId;
   }
   
}
