package hero.entity;

import hero.interfaces.BnNodeLocal;
import hero.interfaces.BnNodePropertyPK;
import hero.interfaces.BnNodePropertyValue;
import hero.interfaces.InvalidValueException;
import hero.interfaces.SequenceGenerator;
import hero.interfaces.SequenceGeneratorHome;
import hero.interfaces.ServiceUnavailableException;

import java.rmi.RemoteException;

import javax.ejb.CreateException;
import javax.ejb.EJBException;
import javax.ejb.EntityBean;
import javax.ejb.EntityContext;
import javax.ejb.RemoveException;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.rmi.PortableRemoteObject;

/**
 * The Entity bean represents a BnNodeProperty
 *
 * @author Oriol Montala
 * @version $Revision: 0.1 $
 *
 * @ejb:bean name="BnNodeProperty"
 *           display-name="BnNode Property Entity"
 *           type="CMP"
 *           jndi-name="ejb/hero/BnNodeProperty"
 *           local-jndi-name="ejb/hero/BnNodeProperty_L"
 *           view-type="both"
 *           cmp-version="2.x"
 *           reentrant="true"       
 *
 * @ejb:env-entry name="SequenceName"
 *                value="BnNodeProperty"
 *
 * @ejb:ejb-ref ejb-name="hero/SequenceGenerator"
 *
 * @ejb:transaction type="Supports"
 *
 * @ejb.pk generate="true" method-intf="both"
 * 
 * @jonas.bean
 *     ejb-name="BnNodeProperty"
 *     jndi-name="ejb/hero/BnNodeProperty"
 * @jonas.jdbc-mapping
 *     jndi-name="jdbc_1"
 *     jdbc-table-name="BnNodeProperty"
 * @jonas.cleanup create
 *
 * @ejb:finder signature="java.util.Collection findAll()" 
 *     query="SELECT object(n) FROM BnNodeProperty n"
 *     transaction-type="Supports"
 * @ejb:finder signature="java.util.Collection findByNode(int nodeId)" view-type="local" 
 *     query="SELECT OBJECT(p) From BnNodeProperty p Where p.bnNode.id = ?1"
 *     transaction-type="Supports"
 * @ejb:finder signature="java.util.Collection findByTheKey(int nodeId, java.lang.String pkey)" view-type="local" 
 *     query="SELECT OBJECT(p) From BnNodeProperty p Where p.bnNode.id = ?1 AND p.theKey=?2"
 *     transaction-type="Supports"
 *
 *
 * @ejb.value-object
 *    match="*"
 *    name="BnNodeProperty"
 *
 *
 * @jboss:table-name table-name="BnNodeProperty"
 *
 * @jboss:create-table create="true"
 *
 * @jboss:remove-table remove="false"
 **/
public abstract class BnNodePropertyBean
   implements EntityBean
{
   
   // -------------------------------------------------------------------------
   // Members
   // -------------------------------------------------------------------------  
   
   public EntityContext mContext;
   
   // -------------------------------------------------------------------------
   // Methods
   // -------------------------------------------------------------------------     
   

   // -------------------------------------------------------------------------
   // Properties (Getters/Setters)
   // -------------------------------------------------------------------------  
   
   /**
   * Retrieve the Property's id.
   *
   * @return Returns an int representing the id of this Property.
   *
   * @ejb:pk-field
   *
   * @ejb:persistence column-name="id"
   * @jonas.cmp-field-jdbc-mapping
   *    field-name="id"
   *    jdbc-field-name="id"
   * @ejb:transaction type="Supports"
   **/
   public abstract int getId();
   
   /**
   * Set the Property's id.
   *
   * @param pId The id of this Property. Is set at creation time.
   * @ejb:transaction type="Required"
   **/
   public abstract void setId( int pId );
   

    
    /** Retrieve the BnNode.
     *
     * @return Returns a BnNodeLocal.
     * @ejb:interface-method view-type="both"
     * @ejb:relation name="nodeprp" role-name="property-by" cascade-delete="yes"
     * @jboss:auto-key-fields
     * @jonas:automatic-pk="true"
     * @ejb:transaction type="Supports"
     **/
    public abstract BnNodeLocal getBnNode();
    
    /**
     * Set the BnNode.
     *
     * @ejb:interface-method view-type="local"
     * @param pNode The BnNode of this Property.
     * @ejb:transaction type="Required"
     **/
    public abstract void setBnNode( BnNodeLocal pNode );



   /**
   * Retrieve the Key value.
   *
   * @return Returns an String representing the Key value of this Property.
   *
   * @ejb:interface-method view-type="both"
   *
   * @ejb:persistence column-name="thekey"
   * @ejb:transaction type="Supports"
   **/
   public abstract String getTheKey();
   
   /**
   * Set the key value.
   * @ejb:interface-method view-type="both"
   * @param key The Key of this Property.
   * @ejb:transaction type="Required"
   **/
   public abstract void setTheKey( String key );

   /**
   * Retrieve the Value value.
   *
   * @return Returns an String representing the Value value of this Property.
   *
   * @ejb:interface-method view-type="both"
   *
   * @ejb:persistence column-name="thevalue"
   * @ejb:transaction type="Supports"
   **/
   public abstract String getTheValue();
   
   /**
   * Set the Value value.
   * @ejb:interface-method view-type="both"
   * @param value The Value value of this Property.
   * @ejb:transaction type="Required"
   **/
   public abstract void setTheValue( String value );

   /**
   * Retrieve the propagation value
   *
   * @return Returns a boolean 
   *
   * @ejb:interface-method view-type="both"
   *
   * @ejb:persistence column-name="propagate"
   * @ejb:transaction type="Supports"
   **/
   public abstract boolean getPropagate();
   
   /**
   * Set the propagation value.
   * @ejb:interface-method view-type="both"
   * @param prop if the property is propagateable
   * @ejb:transaction type="Required"
   **/
   public abstract void setPropagate(boolean prop );

    /**
     * @ejb.interface-method 
     * @ejb:transaction type="Supports"
     */
    public abstract BnNodePropertyValue getBnNodePropertyValue();
    
    /**
     * @ejb.interface-method 
     * @ejb:transaction type="Required"
     */
    public abstract void setBnNodePropertyValue(BnNodePropertyValue v);



   // -------------------------------------------------------------------------
   // Framework Callbacks
   // -------------------------------------------------------------------------  
   
   /**
   * Create a Property based on the supplied Property Value Object.
   *
   *
   * @throws InvalidValueException If one of the values are not correct,
   *                               this will not roll back the transaction
   *                               because the caller has the chance to
   *                               fix the problem and try again
   * @throws EJBException If no new unique ID could be retrieved this will
   *                      rollback the transaction because there is no
   *                      hope to try again
   * @throws CreateException Because we have to do so (EJB spec.)
   *
   * @ejb:create-method view-type="both"
   **/
   public BnNodePropertyPK ejbCreate(BnNodePropertyValue prp )
      throws
         InvalidValueException,
         EJBException,
         CreateException
   {
       setBnNodePropertyValue(prp);

      try {
         // Each title must have a unique id to identify itself within the DB
         this.setId( generateUniqueId() );
      }
      catch( ServiceUnavailableException se ) {
         // The unique id could not be set therefore terminate the transaction
         // by throwing a system exception
         throw new EJBException( se.getMessage() );
      }
      // This is only possible in CMPs. Otherwise return a valid PK.
      return null;
   }

   /**
   * Create a Property based on the supplied  key  and value.
   *
   *
   * @throws InvalidValueException If one of the values are not correct,
   *                               this will not roll back the transaction
   *                               because the caller has the chance to
   *                               fix the problem and try again
   * @throws EJBException If no new unique ID could be retrieved this will
   *                      rollback the transaction because there is no
   *                      hope to try again
   * @throws CreateException Because we have to do so (EJB spec.)
   *
   * @ejb:create-method view-type="both"
   **/
   public BnNodePropertyPK ejbCreate(String key, String value )
      throws
         InvalidValueException,
         EJBException,
         CreateException
   {
       this.setTheKey(key);
       this.setTheValue(value);
       this.setPropagate(true);

      try {
         // Each title must have a unique id to identify itself within the DB
         this.setId( generateUniqueId() );
      }
      catch( ServiceUnavailableException se ) {
         // The unique id could not be set therefore terminate the transaction
         // by throwing a system exception
         throw new EJBException( se.getMessage() );
      }
      // This is only possible in CMPs. Otherwise return a valid PK.
      return null;
   }

   public void setEntityContext( EntityContext lContext )
   {
      mContext = lContext;
   }
   
   public void unsetEntityContext()
   {
      mContext = null;
   }
   
   public void ejbActivate()
   {
   }
   
   public void ejbPassivate()
   {
   }
   
   public void ejbLoad()
   {
   }
   
   public void ejbStore()
   {
   }
   
   public void ejbRemove()
      throws
         RemoveException
   {
   }


   /**
   * Retrive a unique creation id to use for this bean.  This will end up
   * demarcating this bean from others when it is stored as a record
   * in the database.
   *
   * @return Returns an integer that can be used as a unique creation id.
   *
   * @throws ServiceUnavailableException Indicating that it was not possible
   *                                     to retrieve a new unqiue ID because
   *                                     the service is not available
   **/
   private int generateUniqueId()
      throws ServiceUnavailableException
   {
      int lUniqueId = -1;
      try {
         Context lContext = new InitialContext();
         
         String lSequenceName = (String) lContext.lookup( 
            "java:comp/env/SequenceName" 
         );
         SequenceGeneratorHome lHome = (SequenceGeneratorHome) PortableRemoteObject.narrow(
            lContext.lookup(
               "java:comp/env/ejb/hero/SequenceGenerator"
            ),
            SequenceGeneratorHome.class
         );
         SequenceGenerator lBean = (SequenceGenerator) lHome.create();
         lUniqueId = lBean.getNextNumber( lSequenceName );
         lBean.remove();  
      }
      catch ( NamingException ne ) {
         throw new ServiceUnavailableException( "Naming lookup failure: " + ne.getMessage() );
      }
      catch ( CreateException ce ) {
         throw new ServiceUnavailableException( "Failure while creating a generator session bean: " + ce.getMessage() );
      }
      catch ( RemoveException re ) {
         // When the Bean cannot be removed after a while it will be taken back by the container
         // therefore ignore this exception
      }
      catch ( RemoteException rte ) {
         throw new ServiceUnavailableException( "Remote exception occured while accessing generator session bean: " +  rte.getMessage() );
      }
      
      return lUniqueId;
   }
   
}
