package hero.session;
/*
* 01/02/2004 - 16:36:12
*
* UserSessionEJB.java - 
* Copyright (C) 2004 Ecoo Team
* valdes@loria.fr
* 
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
import hero.interfaces.BnNodeLocal;
import hero.interfaces.BnNodeLocalHome;
import hero.interfaces.BnNodeUtil;
import hero.interfaces.BnNodeValue;
import hero.interfaces.BnProjectLocal;
import hero.interfaces.BnProjectLightValue;
import hero.interfaces.BnProjectLocalHome;
import hero.interfaces.BnProjectUtil;
import hero.interfaces.BnUserLightValue;
import hero.interfaces.BnUserLocal;
import hero.interfaces.BnUserLocalHome;
import hero.interfaces.BnUserPropertyLocal;
import hero.interfaces.BnUserPropertyLocalHome;
import hero.interfaces.BnUserPropertyUtil;
import hero.interfaces.BnUserValue;
import hero.interfaces.EngineLocal;
import hero.interfaces.EngineLocalHome;
import hero.interfaces.InvalidValueException;
import hero.interfaces.ProjectSessionLocal;
import hero.interfaces.ProjectSessionLocalHome;
import hero.interfaces.ProjectSessionUtil;
import hero.interfaces.JMSServicesSessionUtil;
import hero.interfaces.JMSServicesSessionLocalHome;
import hero.interfaces.JMSServicesSessionLocal;
import hero.util.EngineException;
import hero.util.HeroException;
import hero.util.EventConstants;
import hero.interfaces.Constants;


import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Iterator;

import javax.ejb.CreateException;
import javax.ejb.EJBException;
import javax.ejb.FinderException;
import javax.ejb.RemoveException;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;

import org.apache.log4j.Category;

/**
 *
 * The User Session Bean, is an stateful session bean that provides the user API to get information on
 * Todo list and started activities and to produce events on activities (start, terminate, cancel).<br>
 * This Session Bean is based on Engine Session Bean: a recursive implementation that manage the previous execution operations
 * and propagates the activity state changes to the activities that are connected to this one.
 *<br><br>
 * The User Session Bean API provides information about user projects and activites (project list,
 * todo list and activity list) and also useful information about project instances or user
 * preferences. With this API users can performs his task/activities by using start, terminate and
 * cancel methods and also terminates workflow processes.
 * <br><br>
 * <strong>The following lines shows a sample code to use this API in your application:<br><br></strong>
 * <br>
 * First of all you have to import the User Session files:<br>
 * <br>
 * import hero.interfaces.UserSessionLocalHome;<br>
 * import hero.interfaces.UserSessionLocal;<br>
 * import hero.interfaces.UserSessionHome;<br>
 * import hero.interfaces.UserSession;<br>
 * import hero.interfaces.UserSessionUtil;<br>
 * <br>
 * Now, it is time to create the User Session instance:<br>  
 * <br>
 * Like this if you want to use local interfaces:<br><br>
 * UserSessionLocalHome userh = (UserSessionLocalHome)hero.interfaces.UserSessionUtil.getLocalHome();<br>
 * UserSessionLocal usersession = userh.create();<br>
 * <br>
 * or like this if you use remote interfaces:<br>  
 * <br>
 * UserSessionHome userh = (UserSessionHome)hero.interfaces.UserSessionUtil.getHome();<br>
 * UserSession usersession = userh.create();<br>
 * <br>
 * <br>
 * after that, you can init the User Session API for this user by using the init method:<br><br>
 * usersession.init();<br>
 * <br>
 * Now you can call all User Sessions methods...  
 * 
 *
 * @ejb:bean name="UserSession"
 *           display-name="UserSession Bean"
 *           type="Stateless"
 *           transaction-type="Container"
 *           jndi-name="ejb/hero/UserSession"
 *           local-jndi-name="ejb/hero/UserSession_L"
 *
 * @ejb:ejb-ref ejb-name="BnUser"
 *              ref-name="myhero/BnUser"
 * @ejb.permission role-name="Admin,user"
 *
 * @jonas.bean
 *      ejb-name="UserSession"
 *      jndi-name="ejb/hero/UserSession"
 *
 * 
 * 
 * @copyright INRIA
 * @author Miguel Valdes
 **/

public class UserSessionBean implements SessionBean, EventConstants {

	// -------------------------------------------------------------------------
	// Static
	// -------------------------------------------------------------------------
	// Utility variable
	private static final Category log = Category.getInstance(UserSessionBean.class);

	// -------------------------------------------------------------------------
	// Members 
	// -------------------------------------------------------------------------

	private SessionContext mContext;

	// -------------------------------------------------------------------------
	// Methods
	// -------------------------------------------------------------------------

	/**
	 * Get user project list. Workflow processes associated to this user
	 * @return a Collection of BnProjectLightValue objects - the projects list of the user
	 * @throws HeroException
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 *
	 **/
	public Collection getProjectList() throws HeroException {
		Collection pjts = new ArrayList();
		Collection pva;
		try{
			BnUserLocalHome userhome = hero.interfaces.BnUserUtil.getLocalHome();
			BnUserLocal mUser = userhome.findByName(mContext.getCallerPrincipal().getName());
			log.debug("getProjectList");
			//Collection projects = null;
			log.debug("before getprojects mUser " + mUser);
			//projects = mUser.getBnProjects();
			BnUserValue uv = mUser.getBnUserValue();
			log.debug("BnUser Value=" + uv);
			pva = new ArrayList(Arrays.asList(uv.getBnProjects()));
			log.debug("Projects " + pva);
		} catch (javax.naming.NamingException ne) {
		    throw new HeroException(ne.getMessage());
	    } catch (javax.ejb.FinderException fe) {
		    throw new HeroException(mContext.getCallerPrincipal().getName() + " does not exists");
		}
		return pva;
	}

	/**
	 * Get project list names for this user
	 * @return a String Collection - the projects list names of the user
	 * @throws HeroException
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 *
	 **/
	public Collection getProjectListNames() throws HeroException {
		log.debug("getProjectListNames");
		//Collection projects = null;
		Collection projectsNames;
		try{
		  BnUserLocalHome userhome = hero.interfaces.BnUserUtil.getLocalHome();
		  BnUserLocal mUser = userhome.findByName(mContext.getCallerPrincipal().getName());
		  log.debug("before getprojects mUser " + mUser);
		  //projects = mUser.getBnProjects();
		  BnUserValue uv = mUser.getBnUserValue();
		  log.debug("BnUser Value=" + uv);
		  Collection pva = new ArrayList(Arrays.asList(uv.getBnProjects()));
		  projectsNames = new ArrayList();
		  Iterator i = pva.iterator();
		  while (i.hasNext())
		     projectsNames.add(((hero.interfaces.BnProjectLightValue) i.next()).getName());
		} catch (javax.naming.NamingException ne) {
	      throw new HeroException(ne.getMessage());
		} catch (javax.ejb.FinderException fe) {
		  throw new HeroException(mContext.getCallerPrincipal().getName() + " does not exists");
		}
		return projectsNames;
	}


	/**
	 * Get user instances list. This method is equivalent to getProjectList but
	 * it only returns the current instances of the user.
	 * @return a Collection of BnProjectLightValue objects - the projects list of the user
	 * @throws HeroException
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 *
	 **/
	public Collection getInstancesList() throws HeroException {
		log.debug("getInstancesList");
		Collection instances = new ArrayList();
		Collection pva = this.getProjectList();
		Iterator ps = pva.iterator();
		
		while (ps.hasNext())
		{
			BnProjectLightValue pro = (BnProjectLightValue)ps.next();
			ProjectSessionLocalHome pHome=null;
			try {
				pHome = hero.interfaces.ProjectSessionUtil.getLocalHome();
			} catch (javax.naming.NamingException be) {
				throw new HeroException(be.getMessage());
			}
			try {
			  ProjectSessionLocal pl = pHome.create();
			  pl.initProject(pro.getName());
			  Collection names = pl.getUserRolesInProjectNames(mContext.getCallerPrincipal().getName());
	    	  if (names.contains(Constants.ADMIN))
		     	instances.add(pro);
			} catch (javax.ejb.CreateException fe) {
				throw new HeroException("Project does not exist");
			}
		}
		log.debug("Instances " + pva);
		return instances;
	}

	/**
	 * Get instances list names for this user. This method is equivalent to getProjectListNames but
	 * it only returns the current instances of the user.
	 * @return a String Collection - the projects list names of the user
	 * @throws HeroException
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 *
	 **/
	public Collection getInstancesListNames() throws HeroException {
		log.debug("getInstancesListNames");
		Collection instancesNames = new ArrayList();
		Collection pva = this.getProjectListNames();
		Iterator i = pva.iterator();
		while (i.hasNext())
		{
			String pro = (String)i.next();
			ProjectSessionLocalHome pHome=null;
			try {
				pHome = hero.interfaces.ProjectSessionUtil.getLocalHome();
			} catch (javax.naming.NamingException be) {
				throw new HeroException(be.getMessage());
			}
			try {
			  ProjectSessionLocal pl = pHome.create();
			  pl.initProject(pro);
			  Collection names = pl.getUserRolesInProjectNames(mContext.getCallerPrincipal().getName());
			  if (names.contains(Constants.ADMIN))
				instancesNames.add(pro);
			} catch (javax.ejb.CreateException fe) {
				throw new HeroException("Instance does not exist");
			}
		}
		return instancesNames;
	}



	/**
	 * Delete a Workflow project
	 * @param projectName the name of the project
	 * @throws HeroException
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 *
	 **/
	public void removeProject(String projectName) throws HeroException {
		log.debug("removeProject projectName=" + projectName);
		BnProjectLocalHome pHome;
		JMSServicesSessionLocal jms;
		try{
		  JMSServicesSessionLocalHome home = JMSServicesSessionUtil.getLocalHome();
		  jms=home.create();
		}catch(Exception e){throw new EngineException(e.getMessage());}
		try {
			pHome = hero.interfaces.BnProjectUtil.getLocalHome();
		} catch (javax.naming.NamingException be) {
			throw new HeroException(be.getMessage());
		}
		try {
			BnProjectLocal pl = pHome.findByName(projectName);
			String userName = mContext.getCallerPrincipal().getName();
			if (userName.equals(pl.getCreator()))  // Only project creator must delete the project
			{
			  Collection nodes = pl.getBnNodes();
			  Iterator nds = nodes.iterator();
			  while (nds.hasNext())            // Verifies if the project has any subProcesses
			  {
				BnNodeLocal nd = (BnNodeLocal)nds.next();
				if (nd.getType()==Constants.Nd.SUB_PROCESS_NODE)
				   throw new HeroException("Error in delete project: You have to delete subProcesses first !!");
			  }
			  if (pl.getParent().equals(pl.getName()))
			  {
				pl.remove();
				jms.sendProjectEvent(DELETEPROJECT, projectName);
			  }
			  else
				throw new HeroException("Error in delete project: You have to delete subProcess activity of this project !!");
			}
			else
				throw new HeroException("Error in delete project: only the project creator can removed it !!");
		} catch (javax.ejb.FinderException fe) {
			throw new HeroException("User does not exist");
		} catch (RemoveException re) {
			throw new HeroException("Error in delete project");
	}

	}


	/**
	* Obtain all user activities from specific project (ready and anticipable state)
	* @param projectName the name of the project
	* @return a String Collection - the list of todo activities ot the user for a project
	* @throws HeroException
	* @ejb:interface-method view-type="both"
	* @ejb:transaction type="Supports"
	*
	**/
	public Collection getToDoList(String projectName) throws HeroException {
		log.debug("getToDoList projectName=" + projectName);
		BnProjectLocalHome projhome;
		BnProjectLocal project;
		BnNodeLocalHome nodehome;
		Collection ready;
		try {
			projhome = BnProjectUtil.getLocalHome();
			nodehome = BnNodeUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {
			throw new HeroException(ne.getMessage());
		}
		try {
			project = projhome.findByName(projectName);
		} catch (FinderException fe) {
			throw new HeroException(
				"BnProject " + projectName + " does not exists");
		}
		log.debug("getToDoList -end projectName=" + projectName);
		try {
			ready = nodehome.findUserReady(project.getId(), mContext.getCallerPrincipal().getName());
		} catch (FinderException fe) {
			log.debug("Empty Todo List for BnUser " + mContext.getCallerPrincipal().getName());
			ready = new ArrayList();
		}
		ArrayList result = new ArrayList();
		for (Iterator i = ready.iterator(); i.hasNext();) {
			BnNodeLocal nd = (BnNodeLocal) i.next();
			log.debug("NODE NAME TODO " + nd.getName());
			result.add(nd.getName());
		}
		return result;
	}

	/**
	 * Obtain all user activities from specific project (executing and anticipating state)
	 *
	 * @param projectName the name of the project
	 * @return a String Collectio - the list of active activities ot the user for a project
	 * @throws HeroException
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 *
	 **/
	public Collection getActivityList(String projectName)
		throws HeroException {
		BnProjectLocalHome projhome;
		BnProjectLocal project;
		BnNodeLocalHome nodehome;
		Collection executing;
		log.debug("getActivityList -begin projectName=" + projectName);
		try {
			projhome = BnProjectUtil.getLocalHome();
			nodehome = BnNodeUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {
			throw new HeroException(ne.getMessage());
		}
		try {
			project = projhome.findByName(projectName);
		} catch (FinderException fe) {
			throw new HeroException(
				"BnProject " + projectName + " does not exists");
		}
		try {
			executing =
				nodehome.findUserExecuting(project.getId(), mContext.getCallerPrincipal().getName());
		} catch (FinderException fe) {
			log.debug("Empty Activity List for BnUser " + mContext.getCallerPrincipal().getName());
			executing = new ArrayList();
		}
		ArrayList result = new ArrayList();
		for (Iterator i = executing.iterator(); i.hasNext();) {
			BnNodeLocal nd = (BnNodeLocal) i.next();
			log.debug("NODE NAME Executing " + nd.getName());
			result.add(nd.getName());
		}
		return result;
	}
		
	/**
	 * Get workflow instances names of this project
	 *
	 * @param projectName the name of the project
	 * @return a String Collection - the names of project instances
	 * @throws HeroException
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 *
	 **/
		
	public Collection getProjectInstancesNames(String projectName) throws HeroException {
		log.debug("getProjectInstancesNames: projectName = "+projectName);
		ArrayList result = new ArrayList();
		BnProjectLocalHome projhome;
		try {
			projhome = hero.interfaces.BnProjectUtil.getLocalHome();
			Collection instances = projhome.findInstancesByProject(projectName+"_instance%");
			for (Iterator i = instances.iterator(); i.hasNext();) {
				BnProjectLocal pl = (BnProjectLocal) i.next();
				result.add(pl.getName());
			}
			return (result);
		} catch (javax.naming.NamingException ne) {
			throw new HeroException(ne.getMessage());
		} catch (javax.ejb.FinderException fe) {
			throw new HeroException(fe.getMessage());
		}			
	}
	
	/**
	 * Get Workflow instances of this project
	 *
	 * @return a BnProjectValue Collection - list of project instances
	 * @param projectName the name of the project
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 *
	 **/
		
	public Collection getProjectInstances(String projectName) throws HeroException {
		log.debug("getInstances: projectName = "+projectName);
		ArrayList result = new ArrayList();
		BnProjectLocalHome projhome;
		try {
			projhome = hero.interfaces.BnProjectUtil.getLocalHome();
			Collection instances = projhome.findInstancesByProject(projectName+"_instance%");
			for (Iterator i = instances.iterator(); i.hasNext();) {
				BnProjectLocal pl = (BnProjectLocal) i.next();
				result.add(pl.getBnProjectValue());
			}
			return (result);
		} catch (javax.naming.NamingException ne) {
			throw new HeroException(ne.getMessage());
		} catch (javax.ejb.FinderException fe) {
			throw new HeroException(fe.getMessage());
		}			
	}

	/**
	 * Get Workflow instances from a property
	 *
	 * @return a BnProjectValue Collection - list of project instances
	 * @param projectName the name of the project
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 *
	 **/

	public Collection getInstancesByProperty(String key, String value)
		throws HeroException {
		log.debug("getInstancesByProperty: value = " + value);
		ArrayList result = new ArrayList();
		BnProjectLocalHome projhome;
		try {
			projhome = hero.interfaces.BnProjectUtil.getLocalHome();
			Collection instances = projhome.findProjectsByProperty(key, value);
			for (Iterator i = instances.iterator(); i.hasNext();) {
				BnProjectLocal pl = (BnProjectLocal) i.next();
				if (pl.getName().matches(".*_instance.*"))
					result.add(pl.getBnProjectValue());
			}
			return (result);
		} catch (javax.naming.NamingException ne) {
			throw new HeroException(ne.getMessage());
		} catch (javax.ejb.FinderException fe) {
			throw new HeroException(fe.getMessage());
		}
	}

	/**
	 * Get Workflow instances from a property
	 *
	 * @return a String Collection - list of project instances
	 * @param projectName the name of the project
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 *
	 **/

	public Collection getInstancesByPropertyNames(String key, String value)
		throws HeroException {
		log.debug("getInstancesByPropertyNames: value = " + value);
		ArrayList result = new ArrayList();
		BnProjectLocalHome projhome;
		try {
			projhome = hero.interfaces.BnProjectUtil.getLocalHome();
			Collection instances = projhome.findProjectsByProperty(key, value);
			for (Iterator i = instances.iterator(); i.hasNext();) {
				BnProjectLocal pl = (BnProjectLocal) i.next();
				if (pl.getName().matches(".*_instance.*"))
					result.add(pl.getName());

			}
			return (result);
		} catch (javax.naming.NamingException ne) {
			throw new HeroException(ne.getMessage());
		} catch (javax.ejb.FinderException fe) {
			throw new HeroException(fe.getMessage());
		}
	}


	/**
	 * Tries to start an activity (when activity state is ready or anticipable)
	 * @param projectName the name of the project
	 * @param nodeName the name of the node
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 **/

	public void startActivity(String projectName, String nodeName)
		throws HeroException {
		log.debug("startActivity projectName="+ projectName+ " nodeName="+ nodeName);
		EngineLocalHome enghome;
		try {
			enghome = hero.interfaces.EngineUtil.getLocalHome();
		    EngineLocal engine = enghome.create();
			engine.startActivity(projectName, nodeName);
			engine.remove();
		} catch (javax.naming.NamingException ne) {
			throw new HeroException(ne.getMessage());
		} catch (RemoveException r) {
			throw new HeroException("Cannot remove Engine bean for "+ projectName+ " "+ r.getMessage());
		} catch (CreateException c) {
			throw new HeroException("Cannot create Engine bean for "+ projectName+ " "+ c.getMessage());
		}
		log.debug("startActivity -end projectName="+ projectName+ " nodeName="+ nodeName);
	}

	/**
	 * Tries to terminate an activity (when activity state is executing or anticipating)
	 * @param projectName the name of the project
	 * @param nodeName the name of the node
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 **/
	public void terminateActivity(String projectName, String nodeName)
		throws HeroException {
		log.debug("terminateActivity projectName="+ projectName+ " nodeName="+ nodeName);
		EngineLocalHome enghome;
		try {
			enghome = hero.interfaces.EngineUtil.getLocalHome();
			EngineLocal engine = enghome.create();
			engine.terminateActivity(projectName, nodeName);
			engine.remove();
		} catch (RemoveException r) {
			throw new HeroException("Cannot remove Engine bean for "+ projectName+ " "+ r.getMessage());			
		} catch (javax.naming.NamingException ne) {
			throw new HeroException(ne.getMessage());
		} catch (CreateException c) {
			throw new HeroException("Cannot create Engine bean for " + projectName);
		}
		log.debug("terminateActivity -end projectName="+ projectName+ " nodeName="+ nodeName);
	}

	/**
	 * Tries to cancel an activity (when activity is executing or anticipating)
	 * @param projectName the name of the project
	 * @param nodeName the name of the node
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 **/
	public void cancelActivity(String projectName, String nodeName)
		throws HeroException {
		log.debug("cancelActivity projectName="+ projectName+ " nodeName="+ nodeName);
		EngineLocalHome enghome;
		try {
			enghome = hero.interfaces.EngineUtil.getLocalHome();
			EngineLocal engine = enghome.create();
			engine.cancelActivity(projectName, nodeName);
			engine.remove();
		} catch (RemoveException r) {
			throw new HeroException("Cannot remove Engine bean for "+ projectName+ " "+ r.getMessage());
		} catch (javax.naming.NamingException ne) {
			throw new HeroException(ne.getMessage());
		} catch (CreateException c) {
			throw new EJBException("Cannot create Engine bean for " + projectName);
		}
		log.debug("cancelActivity -end projectName="+ projectName+ " nodeName="+ nodeName);
	}
	/**
	 * Tries to terminate a project (only when all project activities are terminated)
	 * @param projectName the name of the project
	 * @throws HeroException
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 **/
	public void terminate(String projectName) throws HeroException {
		log.debug("terminate projectName=" + projectName);
		EngineLocalHome enghome;
		try {
			enghome = hero.interfaces.EngineUtil.getLocalHome();
			EngineLocal engine = enghome.create();
			engine.terminate(projectName);
			engine.remove();
		} catch (RemoveException r) {
			throw new HeroException("Cannot remove Engine bean for "+ projectName+ " "+ r.getMessage());
		} catch (javax.naming.NamingException ne) {
			throw new HeroException(ne.getMessage());
		} catch (CreateException c) {
			throw new EJBException("Cannot create Engine bean for " + projectName);
		} catch (EngineException e) {
			throw new HeroException("Cannot terminate project " + e.getMessage());
		}
		log.debug("terminate");
	}

	/**
	 * Get Node Value from a specific project
	 * @return a NodeValue object - get node information
	 * @param projectName the name of the project
	 * @param nodeName the name of the node
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 **/
	public BnNodeValue getNode(String projectName, String nodeName)
		throws HeroException {
		log.debug("getNode projectName=" + projectName + " nodeName=" + nodeName);
		try {
			ProjectSessionLocalHome projectseshome =ProjectSessionUtil.getLocalHome();
			ProjectSessionLocal project = projectseshome.create();
			project.initProject(projectName);
			if (!checkRights(project))
				throw new HeroException("User is not member of the project");
			return project.getNodeValue(nodeName);
		} catch (CreateException f) {
			throw new HeroException("Project does not exists " + projectName);
		} catch (javax.naming.NamingException r) {
			throw new HeroException("Cannot access to project " + r.getMessage());
		}
	}

	/**
	 * Get the user name
	 * 
	 * @return a String - get user name
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 **/
	public String getUser() {
		return mContext.getCallerPrincipal().getName();
	}

	/**
	 * Get the user light value
	 * 
	 * @return a BnUserLightValue object - get user information
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 **/
	public BnUserLightValue getUserLightValue() throws HeroException{
		try{
			BnUserLocalHome userhome = hero.interfaces.BnUserUtil.getLocalHome();
			BnUserLocal mUser = userhome.findByName(mContext.getCallerPrincipal().getName());
			return mUser.getBnUserLightValue();
		} catch (javax.naming.NamingException ne) {
			throw new HeroException(ne.getMessage());
		} catch (javax.ejb.FinderException fe) {
			throw new HeroException(mContext.getCallerPrincipal().getName() + " does not exists");
		}
	}
	
	/**
	 * Get the mail of this user from Bonita database.
	 *
	 * @param userName the name of the user
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 *
	**/
	public String getUserMail(String userName) throws HeroException {
		log.debug("getUserMail: userName = " + userName);
		try {
			BnUserLocalHome userhome = hero.interfaces.BnUserUtil.getLocalHome();
			BnUserLocal user = userhome.findByName(userName);
			return (user.getEmail());
		} catch (javax.naming.NamingException ne) {
				throw new HeroException(ne.getMessage());
		} catch (FinderException fe) {
			throw new HeroException(fe.getMessage());
		}
	}
	
	/**
	 * Set the mail of this user into Bonita database.
	 *
	 * @param userName the name of the user
	 * @param userName the mail of the user
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	**/
	public void setUserMail(String userName, String mail) throws HeroException {
		log.debug("setUserMail: userName = " + userName+" mail = "+mail);
		try {
			BnUserLocalHome userhome = hero.interfaces.BnUserUtil.getLocalHome();
			BnUserLocal user = userhome.findByName(userName);
			user.setEmail(mail);
		} catch (javax.naming.NamingException ne) {
			throw new HeroException(ne.getMessage());
		} catch (FinderException fe) {
			throw new HeroException(fe.getMessage());
		}
	}
	

	/**
	 * Get the user password
	 *
	 * @return a String - get user password
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 **/
	public String getUserPassword() throws HeroException{
		try{
			BnUserLocalHome userhome = hero.interfaces.BnUserUtil.getLocalHome();
			BnUserLocal mUser = userhome.findByName(mContext.getCallerPrincipal().getName());
			return mUser.getPassword();
		} catch (javax.naming.NamingException ne) {
			throw new HeroException(ne.getMessage());
		} catch (javax.ejb.FinderException fe) {
			throw new HeroException(mContext.getCallerPrincipal().getName() + " does not exists");
		}
	}

	/**
	  * Get User properties
	  *
	  * @return a BnUserPropertyValue Collection - list of user properties
	  * @ejb:interface-method view-type="both"
	  * @ejb:transaction type="Supports"
	  *
	  **/
	public Collection getUserProperties() throws HeroException {
		Collection prp = new ArrayList();
		BnUserPropertyLocal pl;
		try{
			BnUserLocalHome userhome = hero.interfaces.BnUserUtil.getLocalHome();
			BnUserLocal mUser = userhome.findByName(mContext.getCallerPrincipal().getName());
			Collection pls = mUser.getBnProperties();

			for (Iterator i = pls.iterator(); i.hasNext();) {
				pl = (BnUserPropertyLocal) i.next();
				prp.add(pl.getBnUserPropertyValue());
			}
			return (prp);
		} catch (javax.naming.NamingException ne) {
			throw new HeroException(ne.getMessage());
		} catch (javax.ejb.FinderException fe) {
			throw new HeroException(mContext.getCallerPrincipal().getName() + " does not exists");
		}
	}

	/**
	 * Set a property of a user
	 *
	 * @param key the key of the property
	 * @param value the name of the property
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 *
	 **/

	public void setUserProperty(String key, String value)
		throws HeroException {
		BnUserPropertyLocalHome pHome;
		BnUserPropertyLocal propertyLocal = null;

		log.debug("setUserProperty  key=" + key + " value=" + value);

		try {
			BnUserLocalHome userhome =	hero.interfaces.BnUserUtil.getLocalHome();
			BnUserLocal mUser =	userhome.findByName(mContext.getCallerPrincipal().getName());
			try {
				pHome = BnUserPropertyUtil.getLocalHome();
			} catch (javax.naming.NamingException ne) {
				throw new HeroException(ne.getMessage());
			}
			Collection c = null;
			try {
				c = pHome.findByTheKey(mUser.getId(), key);
			} catch (FinderException fe) {
				throw new HeroException(fe.getMessage());
			}

			if (!c.isEmpty()) {

				propertyLocal = (BnUserPropertyLocal) (c.toArray())[0];
				propertyLocal.setTheValue(value);

			} else {
				try {
					propertyLocal = pHome.create(key, value);
					propertyLocal.setBnUser(mUser);

				} catch (InvalidValueException ie) {
					throw new HeroException(ie.getMessage());
				} catch (javax.ejb.CreateException ce) {
					throw new HeroException(ce.getMessage());
				}

			}

		} catch (javax.naming.NamingException ne) {
			throw new HeroException(ne.getMessage());
		} catch (javax.ejb.FinderException fe) {
			throw new HeroException(
				mContext.getCallerPrincipal().getName() + " does not exists");
		}
	}

	/**
	  * Delete a property of a user
	  *
	  * @param key the key of the property
	  * @ejb:interface-method view-type="both"
	  * @ejb:transaction type="Required"
	  *
	  **/
	public void deleteUserProperty(String key) throws HeroException {
		log.debug("deleteUserProperty key=" + key);

		BnUserPropertyLocalHome pHome;
		BnUserPropertyLocal propertyLocal = null;
		try {
			BnUserLocalHome userhome =	hero.interfaces.BnUserUtil.getLocalHome();
			BnUserLocal mUser =	userhome.findByName(mContext.getCallerPrincipal().getName());
			try {
				pHome = BnUserPropertyUtil.getLocalHome();
			} catch (javax.naming.NamingException ne) {
				throw new HeroException(ne.getMessage());
			}
			Collection c = null;
			try {
				c = pHome.findByTheKey(mUser.getId(), key);
			} catch (FinderException fe) {
				throw new HeroException(fe.getMessage());
			}

			if (!c.isEmpty()) {
				try {
					propertyLocal = (BnUserPropertyLocal) (c.toArray())[0];
					propertyLocal.remove();
				} catch (RemoveException rm) {
					throw new HeroException(rm.getMessage());
				}
			} else {
				throw new HeroException("Property not found :" + key);
			}

		} catch (javax.naming.NamingException ne) {
			throw new HeroException(ne.getMessage());
		} catch (javax.ejb.FinderException fe) {
			throw new HeroException(
				mContext.getCallerPrincipal().getName() + " does not exists");
		}

	}

	/**
	* Creates the User Session Bean. This method is the first one to invoke in order to
	* use UserSession API. If the user is not authorized this method throws an exception.
	*
	* @throws CreateException 
	*
	* @ejb:create-method view-type="both"
	**/
	public void ejbCreate() throws CreateException {
	}

	/**
    * Internal Enterprise Java Beans method. 
	**/

	public void setSessionContext(final javax.ejb.SessionContext context) {
		mContext = context;
	}

	/**
	* Internal Enterprise Java Beans method. 
	**/
	public void ejbRemove() {
	}

	/**
	* Internal Enterprise Java Beans method. 
	**/
	public void ejbActivate() {
	}

	/**
	* Internal Enterprise Java Beans method. 
	**/
	public void ejbPassivate() {
	}

	// utility methods
	private boolean checkRights(ProjectSessionLocal ps) throws HeroException {
		// check that current user is a member of the project
		return ps.containsUser(mContext.getCallerPrincipal().getName());
	}
}
