/*
*
* BonitaServiceLocator.java - 
* Copyright (C) 2002 Ecoo Team
* charoy@loria.fr
* 
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

package hero.util;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.rmi.PortableRemoteObject;

import org.apache.log4j.Category;

public class BonitaServiceLocator {
    private static BonitaServiceLocator msl;
    private static Context initial;

    // Utility variable
    private static final Category log = Category.getInstance(BonitaServiceLocator.class);


    private BonitaServiceLocator() throws BonitaServiceException {
	log.debug("Initialisation");
	try {
	    initial=new InitialContext();
	} catch (javax.naming.NamingException ne) {
	    throw new BonitaServiceException(ne.getMessage());
	}
    }

    static public BonitaServiceLocator getInstance() throws BonitaServiceException {
	log.debug("getInstance");
	if (msl==null) {
	    msl=new BonitaServiceLocator();
	}
	return msl;
    }

    // Services constant inner class - service objects

    public class Services {
	final public static int PROJECT=0;
	final public static int PROJECTLOCAL=1;
	final public static int NODE=2;
	final public static int NODELOCAL=3;
	final public static int USER=4;
	final public static int USERLOCAL=5;
	final public static int SEQUENCEGENERATOR=6;
	final public static int EDGE=7;
	final public static int EDGELOCAL=8;
	final public static int ROLE=9;
	final public static int ROLELOCAL=10;
	final public static int USERSESSION=11;
	final public static int PROJECTSESSION=12;
    }

    // EJB related constants
    final static Class PROJECT_CLASS = hero.interfaces.BnProjectHome.class;
    final static String PROJECT_NAME = "ejb/hero/BnProject";
    final static Class PROJECTLOCAL_CLASS = hero.interfaces.BnProjectLocalHome.class;
    final static String PROJECTLOCAL_NAME = "ejb/hero/BnProjectLocal";

    final static Class NODE_CLASS = hero.interfaces.BnNodeHome.class;
    final static String NODE_NAME = "ejb/hero/BnNode";
    final static Class NODELOCAL_CLASS = hero.interfaces.BnNodeLocalHome.class;
    final static String NODELOCAL_NAME = "ejb/hero/BnNodeLocal";

    final static Class USER_CLASS = hero.interfaces.BnUserHome.class;
    final static String USER_NAME = "ejb/hero/BnUser";
    final static Class USERLOCAL_CLASS = hero.interfaces.BnUserLocalHome.class;
    final static String USERLOCAL_NAME = "ejb/hero/BnUserLocal";

    final static Class EDGE_CLASS = hero.interfaces.BnEdgeHome.class;
    final static String EDGE_NAME = "ejb/hero/BnEdge";
    final static Class EDGELOCAL_CLASS = hero.interfaces.BnEdgeLocalHome.class;
    final static String EDGELOCAL_NAME = "ejb/hero/BnEdgeLocal";

    final static Class ROLE_CLASS = hero.interfaces.BnRoleHome.class;
    final static String ROLE_NAME = "ejb/hero/BnRole";
    final static Class ROLELOCAL_CLASS = hero.interfaces.BnRoleLocalHome.class;
    final static String ROLELOCAL_NAME = "ejb/hero/BnRoleLocal";

    final static Class USERSESSION_CLASS = hero.interfaces.UserSessionHome.class;
    final static String USERSESSION_NAME = "ejb/hero/UserSession";
    final static Class PROJECTSESSION_CLASS = hero.interfaces.ProjectSessionHome.class;
    final static String PROJECTSESSION_NAME = "ejb/hero/ProjectSession";

    final static Class SEQUENCEGENERATOR_CLASS = hero.interfaces.SequenceGeneratorHome.class;
    final static String SEQUENCEGENERATOR_NAME = "ejb/hero/SequenceGenerator";

    // Database related constants
    final static String projectDBName = "java:/MySqlDS";

    // Returns the class for the required service

    static private Class getServiceClass(int service) {
	switch(service) {
	case Services.PROJECT: return PROJECT_CLASS;
	case Services.PROJECTLOCAL: return PROJECTLOCAL_CLASS;
	case Services.NODE: return NODE_CLASS;
	case Services.NODELOCAL: return NODELOCAL_CLASS;
	case Services.USER: return USER_CLASS;
	case Services.USERLOCAL: return USERLOCAL_CLASS;
	case Services.SEQUENCEGENERATOR: return SEQUENCEGENERATOR_CLASS;
	case Services.EDGE: return EDGE_CLASS;
	case Services.EDGELOCAL: return EDGELOCAL_CLASS;
	case Services.ROLE: return ROLE_CLASS;
	case Services.ROLELOCAL: return ROLELOCAL_CLASS;
	case Services.USERSESSION: return USERSESSION_CLASS;
	case Services.PROJECTSESSION: return PROJECTSESSION_CLASS;
	}
	return null;
    }

    // Returns the JNDI name for the required service

    static private  String getServiceName(int service) {
	switch (service) {
	case Services.PROJECT: return PROJECT_NAME;
	case Services.PROJECTLOCAL: return PROJECTLOCAL_NAME;
	case Services.NODE: return NODE_NAME;
	case Services.NODELOCAL: return NODELOCAL_NAME;
	case Services.USER: return USER_NAME;
	case Services.USERLOCAL: return USERLOCAL_NAME;
	case Services.EDGE: return EDGE_NAME;
	case Services.EDGELOCAL: return EDGELOCAL_NAME;
	case Services.ROLE: return ROLE_NAME;
	case Services.ROLELOCAL: return ROLELOCAL_NAME;
	case Services.USERSESSION: return USERSESSION_NAME;
	case Services.PROJECTSESSION: return PROJECTSESSION_NAME;
	case Services.SEQUENCEGENERATOR: return SEQUENCEGENERATOR_NAME;
	}
	return null;
    }

    // returns a EJBHome for the given service, using the JNDI name
    // and the Class for the EJBHome

    public Object getHome(int s) throws BonitaServiceException {
	log.debug("getHome: name="+getServiceName(s));
	Object home=null;
	try {
	    Context initial = new InitialContext();
	    String serv=getServiceName(s);
	    Object objref = initial.lookup(serv);
	    Object obj=PortableRemoteObject.narrow(objref,getServiceClass(s));
	    home=obj;
	} catch (NamingException ex) {
	    throw new BonitaServiceException("ServiceException");
	} catch (Exception ex) {
	    System.out.println("ServiceLocator Exception");
	    ex.printStackTrace();
	}

	return home;
    }



	    
}
